@@ -28,66 +28,224 @@ import asyncio
 import itertools
 import logging
 
-import websockets
+import aiohttp
 
 from .state import AutoShardedConnectionState
 from .client import Client
+from .backoff import ExponentialBackoff
 from .gateway import *
-from .errors import ClientException, InvalidArgument
+from .errors import ClientException, InvalidArgument, HTTPException, GatewayNotFound, ConnectionClosed
 from . import utils
 from .enums import Status
 
 log = logging.getLogger(__name__)
 
+class EventType:
+    close = 0
+    reconnect = 1
+    resume = 2
+    identify = 3
+    terminate = 4
+    clean_close = 5
+
+class EventItem:
+    __slots__ = ('type', 'shard', 'error')
+
+    def __init__(self, etype, shard, error):
+        self.type = etype
+        self.shard = shard
+        self.error = error
+
+    def __lt__(self, other):
+        if not isinstance(other, EventItem):
+            return NotImplemented
+        return self.type < other.type
+
+    def __eq__(self, other):
+        if not isinstance(other, EventItem):
+            return NotImplemented
+        return self.type == other.type
+
+    def __hash__(self):
+        return hash(self.type)
+
 class Shard:
-    def __init__(self, ws, client):
+    def __init__(self, ws, client, queue_put):
         self.ws = ws
         self._client = client
         self._dispatch = client.dispatch
+        self._queue_put = queue_put
         self.loop = self._client.loop
-        self._current = self.loop.create_future()
-        self._current.set_result(None) # we just need an already done future
-        self._pending = asyncio.Event()
-        self._pending_task = None
+        self._disconnect = False
+        self._reconnect = client._reconnect
+        self._backoff = ExponentialBackoff()
+        self._task = None
+        self._handled_exceptions = (
+            OSError,
+            HTTPException,
+            GatewayNotFound,
+            ConnectionClosed,
+            aiohttp.ClientError,
+            asyncio.TimeoutError,
+        )
 
     @property
     def id(self):
         return self.ws.shard_id
 
-    def is_pending(self):
-        return not self._pending.is_set()
+    def launch(self):
+        self._task = self.loop.create_task(self.worker())
 
-    def complete_pending_reads(self):
-        self._pending.set()
+    def _cancel_task(self):
+        if self._task is not None and not self._task.done():
+            self._task.cancel()
 
-    async def _pending_reads(self):
+    async def close(self):
+        self._cancel_task()
+        await self.ws.close(code=1000)
+
+    async def disconnect(self):
+        await self.close()
+        self._dispatch('shard_disconnect', self.id)
+
+    async def _handle_disconnect(self, e):
+        self._dispatch('disconnect')
+        self._dispatch('shard_disconnect', self.id)
+        if not self._reconnect:
+            self._queue_put(EventItem(EventType.close, self, e))
+            return
+
+        if self._client.is_closed():
+            return
+
+        if isinstance(e, OSError) and e.errno in (54, 10054):
+            # If we get Connection reset by peer then always try to RESUME the connection.
+            exc = ReconnectWebSocket(self.id, resume=True)
+            self._queue_put(EventItem(EventType.resume, self, exc))
+            return
+
+        if isinstance(e, ConnectionClosed):
+            if e.code != 1000:
+                self._queue_put(EventItem(EventType.close, self, e))
+                return
+
+        retry = self._backoff.delay()
+        log.error('Attempting a reconnect for shard ID %s in %.2fs', self.id, retry, exc_info=e)
+        await asyncio.sleep(retry)
+        self._queue_put(EventItem(EventType.reconnect, self, e))
+
+    async def worker(self):
+        while not self._client.is_closed():
+            try:
+                await self.ws.poll_event()
+            except ReconnectWebSocket as e:
+                etype = EventType.resume if e.resume else EventType.identify
+                self._queue_put(EventItem(etype, self, e))
+                break
+            except self._handled_exceptions as e:
+                await self._handle_disconnect(e)
+                break
+            except asyncio.CancelledError:
+                break
+            except Exception as e:
+                self._queue_put(EventItem(EventType.terminate, self, e))
+                break
+
+    async def reidentify(self, exc):
+        self._cancel_task()
+        self._dispatch('disconnect')
+        self._dispatch('shard_disconnect', self.id)
+        log.info('Got a request to %s the websocket at Shard ID %s.', exc.op, self.id)
         try:
-            while self.is_pending():
-                await self.poll()
-        except asyncio.CancelledError:
-            pass
-
-    def launch_pending_reads(self):
-        self._pending_task = asyncio.ensure_future(self._pending_reads(), loop=self.loop)
-
-    def wait(self):
-        return self._pending_task
-
-    async def poll(self):
-        try:
-            await self.ws.poll_event()
-        except ResumeWebSocket:
-            log.info('Got a request to RESUME the websocket at Shard ID %s.', self.id)
-            coro = DiscordWebSocket.from_client(self._client, resume=True, shard_id=self.id,
+            coro = DiscordWebSocket.from_client(self._client, resume=exc.resume, shard_id=self.id,
                                                 session=self.ws.session_id, sequence=self.ws.sequence)
-            self._dispatch('disconnect')
-            self.ws = await asyncio.wait_for(coro, timeout=180.0)
+            self.ws = await asyncio.wait_for(coro, timeout=60.0)
+        except self._handled_exceptions as e:
+            await self._handle_disconnect(e)
+        except asyncio.CancelledError:
+            return
+        except Exception as e:
+            self._queue_put(EventItem(EventType.terminate, self, e))
+        else:
+            self.launch()
 
-    def get_future(self):
-        if self._current.done():
-            self._current = asyncio.ensure_future(self.poll(), loop=self.loop)
+    async def reconnect(self):
+        self._cancel_task()
+        try:
+            coro = DiscordWebSocket.from_client(self._client, shard_id=self.id)
+            self.ws = await asyncio.wait_for(coro, timeout=60.0)
+        except self._handled_exceptions as e:
+            await self._handle_disconnect(e)
+        except asyncio.CancelledError:
+            return
+        except Exception as e:
+            self._queue_put(EventItem(EventType.terminate, self, e))
+        else:
+            self.launch()
 
-        return self._current
+class ShardInfo:
+    """A class that gives information and control over a specific shard.
+
+    You can retrieve this object via :meth:`AutoShardedClient.get_shard`
+    or :attr:`AutoShardedClient.shards`.
+
+    .. versionadded:: 1.4
+
+    Attributes
+    ------------
+    id: :class:`int`
+        The shard ID for this shard.
+    shard_count: Optional[:class:`int`]
+        The shard count for this cluster. If this is ``None`` then the bot has not started yet.
+    """
+
+    __slots__ = ('_parent', 'id', 'shard_count')
+
+    def __init__(self, parent, shard_count):
+        self._parent = parent
+        self.id = parent.id
+        self.shard_count = shard_count
+
+    def is_closed(self):
+        """:class:`bool`: Whether the shard connection is currently closed."""
+        return not self._parent.ws.open
+
+    async def disconnect(self):
+        """|coro|
+
+        Disconnects a shard. When this is called, the shard connection will no
+        longer be open.
+
+        If the shard is already disconnected this does nothing.
+        """
+        if self.is_closed():
+            return
+
+        await self._parent.disconnect()
+
+    async def reconnect(self):
+        """|coro|
+
+        Disconnects and then connects the shard again.
+        """
+        if not self.is_closed():
+            await self._parent.disconnect()
+        await self._parent.reconnect()
+
+    async def connect(self):
+        """|coro|
+
+        Connects a shard. If the shard is already connected this does nothing.
+        """
+        if not self.is_closed():
+            return
+
+        await self._parent.reconnect()
+
+    @property
+    def latency(self):
+        """:class:`float`: Measures latency between a HEARTBEAT and a HEARTBEAT_ACK in seconds for this shard."""
+        return self._parent.ws.latency
 
 class AutoShardedClient(Client):
     """A client similar to :class:`Client` except it handles the complications
@@ -126,38 +284,20 @@ class AutoShardedClient(Client):
             elif not isinstance(self.shard_ids, (list, tuple)):
                 raise ClientException('shard_ids parameter must be a list or a tuple.')
 
-        self._connection = AutoShardedConnectionState(dispatch=self.dispatch, chunker=self._chunker,
+        self._connection = AutoShardedConnectionState(dispatch=self.dispatch,
                                                       handlers=self._handlers, syncer=self._syncer,
-                                                      http=self.http, loop=self.loop, **kwargs)
+                                                      hooks=self._hooks, http=self.http, loop=self.loop, **kwargs)
 
         # instead of a single websocket, we have multiple
         # the key is the shard_id
-        self.shards = {}
+        self.__shards = {}
+        self._connection._get_websocket = self._get_websocket
+        self.__queue = asyncio.PriorityQueue()
 
-        def _get_websocket(guild_id):
-            i = (guild_id >> 22) % self.shard_count
-            return self.shards[i].ws
-
-        self._connection._get_websocket = _get_websocket
-
-    async def _chunker(self, guild, *, shard_id=None):
-        try:
-            guild_id = guild.id
-            shard_id = shard_id or guild.shard_id
-        except AttributeError:
-            guild_id = [s.id for s in guild]
-
-        payload = {
-            'op': 8,
-            'd': {
-                'guild_id': guild_id,
-                'query': '',
-                'limit': 0
-            }
-        }
-
-        ws = self.shards[shard_id].ws
-        await ws.send_as_json(payload)
+    def _get_websocket(self, guild_id=None, *, shard_id=None):
+        if shard_id is None:
+            shard_id = (guild_id >> 22) % self.shard_count
+        return self.__shards[shard_id].ws
 
     @property
     def latency(self):
@@ -167,9 +307,9 @@ class AutoShardedClient(Client):
         latency of every shard's latency. To get a list of shard latency, check the
         :attr:`latencies` property. Returns ``nan`` if there are no shards ready.
         """
-        if not self.shards:
+        if not self.__shards:
             return float('nan')
-        return sum(latency for _, latency in self.latencies) / len(self.shards)
+        return sum(latency for _, latency in self.latencies) / len(self.__shards)
 
     @property
     def latencies(self):
@@ -177,7 +317,21 @@ class AutoShardedClient(Client):
 
         This returns a list of tuples with elements ``(shard_id, latency)``.
         """
-        return [(shard_id, shard.ws.latency) for shard_id, shard in self.shards.items()]
+        return [(shard_id, shard.ws.latency) for shard_id, shard in self.__shards.items()]
+
+    def get_shard(self, shard_id):
+        """Optional[:class:`ShardInfo`]: Gets the shard information at a given shard ID or ``None`` if not found."""
+        try:
+            parent = self.__shards[shard_id]
+        except KeyError:
+            return None
+        else:
+            return ShardInfo(parent, self.shard_count)
+
+    @utils.cached_property
+    def shards(self):
+        """Mapping[int, :class:`ShardInfo`]: Returns a mapping of shard IDs to their respective info object."""
+        return { shard_id: ShardInfo(parent, self.shard_count) for shard_id, parent in self.__shards.items() }
 
     async def request_offline_members(self, *guilds):
         r"""|coro|
@@ -210,37 +364,18 @@ class AutoShardedClient(Client):
             sub_guilds = list(sub_guilds)
             await self._connection.request_offline_members(sub_guilds, shard_id=shard_id)
 
-    async def launch_shard(self, gateway, shard_id):
+    async def launch_shard(self, gateway, shard_id, *, initial=False):
         try:
-            coro = websockets.connect(gateway, loop=self.loop, klass=DiscordWebSocket, compression=None)
+            coro = DiscordWebSocket.from_client(self, initial=initial, gateway=gateway, shard_id=shard_id)
             ws = await asyncio.wait_for(coro, timeout=180.0)
         except Exception:
-            log.info('Failed to connect for shard_id: %s. Retrying...', shard_id)
-            await asyncio.sleep(5.0)
-            return await self.launch_shard(gateway, shard_id)
-
-        ws.token = self.http.token
-        ws._connection = self._connection
-        ws._discord_parsers = self._connection.parsers
-        ws._dispatch = self.dispatch
-        ws.gateway = gateway
-        ws.shard_id = shard_id
-        ws.shard_count = self.shard_count
-        ws._max_heartbeat_timeout = self._connection.heartbeat_timeout
-
-        try:
-            # OP HELLO
-            await asyncio.wait_for(ws.poll_event(), timeout=180.0)
-            await asyncio.wait_for(ws.identify(), timeout=180.0)
-        except asyncio.TimeoutError:
-            log.info('Timed out when connecting for shard_id: %s. Retrying...', shard_id)
+            log.exception('Failed to connect for shard_id: %s. Retrying...', shard_id)
             await asyncio.sleep(5.0)
             return await self.launch_shard(gateway, shard_id)
 
         # keep reading the shard while others connect
-        self.shards[shard_id] = ret = Shard(ws, self)
-        ret.launch_pending_reads()
-        await asyncio.sleep(5.0)
+        self.__shards[shard_id] = ret = Shard(ws, self, self.__queue.put_nowait)
+        ret.launch()
 
     async def launch_shards(self):
         if self.shard_count is None:
@@ -254,25 +389,31 @@ class AutoShardedClient(Client):
         self._connection.shard_ids = shard_ids
 
         for shard_id in shard_ids:
-            await self.launch_shard(gateway, shard_id)
+            initial = shard_id == shard_ids[0]
+            await self.launch_shard(gateway, shard_id, initial=initial)
 
-        shards_to_wait_for = []
-        for shard in self.shards.values():
-            shard.complete_pending_reads()
-            shards_to_wait_for.append(shard.wait())
+        self._connection.shards_launched.set()
 
-        # wait for all pending tasks to finish
-        await utils.sane_wait_for(shards_to_wait_for, timeout=300.0)
-
-    async def _connect(self):
+    async def connect(self, *, reconnect=True):
+        self._reconnect = reconnect
         await self.launch_shards()
 
-        while True:
-            pollers = [shard.get_future() for shard in self.shards.values()]
-            done, _ = await asyncio.wait(pollers, return_when=asyncio.FIRST_COMPLETED)
-            for f in done:
-                # we wanna re-raise to the main Client.connect handler if applicable
-                f.result()
+        while not self.is_closed():
+            item = await self.__queue.get()
+            if item.type == EventType.close:
+                await self.close()
+                if isinstance(item.error, ConnectionClosed) and item.error.code != 1000:
+                    raise item.error
+                return
+            elif item.type in (EventType.identify, EventType.resume):
+                await item.shard.reidentify(item.error)
+            elif item.type == EventType.reconnect:
+                await item.shard.reconnect()
+            elif item.type == EventType.terminate:
+                await self.close()
+                raise item.error
+            elif item.type == EventType.clean_close:
+                return
 
     async def close(self):
         """|coro|
@@ -290,11 +431,12 @@ class AutoShardedClient(Client):
             except Exception:
                 pass
 
-        to_close = [asyncio.ensure_future(shard.ws.close(code=1000), loop=self.loop) for shard in self.shards.values()]
+        to_close = [asyncio.ensure_future(shard.close(), loop=self.loop) for shard in self.__shards.values()]
         if to_close:
             await asyncio.wait(to_close)
 
         await self.http.close()
+        self.__queue.put_nowait(EventItem(EventType.clean_close, None, None))
 
     async def change_presence(self, *, activity=None, status=None, afk=False, shard_id=None):
         """|coro|
@@ -339,20 +481,20 @@ class AutoShardedClient(Client):
             status = str(status)
 
         if shard_id is None:
-            for shard in self.shards.values():
+            for shard in self.__shards.values():
                 await shard.ws.change_presence(activity=activity, status=status, afk=afk)
 
             guilds = self._connection.guilds
         else:
-            shard = self.shards[shard_id]
+            shard = self.__shards[shard_id]
             await shard.ws.change_presence(activity=activity, status=status, afk=afk)
             guilds = [g for g in self._connection.guilds if g.shard_id == shard_id]
 
+        activities = () if activity is None else (activity,)
         for guild in guilds:
             me = guild.me
             if me is None:
                 continue
 
-            if activity is not None:
-                me.activities = (activity,)
+            me.activities = activities
             me.status = status_enum
